Module: ProMotion::MapScreenModule
- Included in:
- MapScreen
- Defined in:
- lib/ProMotion/map/map_screen_module.rb
Defined Under Namespace
Modules: MapClassMethods
Class Method Summary collapse
Instance Method Summary collapse
- #add_annotation(annotation) ⇒ Object
- #add_annotations(annotations) ⇒ Object
- #annotation_view(map_view, annotation) ⇒ Object
- #annotations ⇒ Object
- #center ⇒ Object
- #center=(params = {}) ⇒ Object
- #check_annotation_data ⇒ Object
- #clear_annotations ⇒ Object
- #deselect_annotations(animated = false) ⇒ Object
- #hide_user_location ⇒ Object
- #look_up_address(args = {}, &callback) ⇒ Object
- #map ⇒ Object (also: #mapview)
-
#mapView(map_view, didUpdateUserLocation: userLocation) ⇒ Object
Cocoa touch methods #################.
- #region(params) ⇒ Object
- #screen_setup ⇒ Object
- #select_annotation(annotation, animated = true) ⇒ Object
- #select_annotation_at(annotation_index, animated = true) ⇒ Object
- #selected_annotations ⇒ Object
- #set_region(region, animated = true) ⇒ Object
- #set_show_user_location(show) ⇒ Object
- #set_start_position(params = {}) ⇒ Object
- #set_up_start_position ⇒ Object
- #show_user_location ⇒ Object
- #showing_user_location? ⇒ Boolean
-
#type ⇒ Object
Cocoa touch Ruby counterparts #################.
- #type=(type) ⇒ Object
- #update_annotation_data ⇒ Object
- #user_location ⇒ Object
- #view_will_appear(animated) ⇒ Object
-
#zoom_to_fit_annotations(animated = true) ⇒ Object
TODO: Why is this so complex?.
- #zoom_to_user(radius = 0.05, animated = true) ⇒ Object
Class Method Details
.included(base) ⇒ Object
278 279 280 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 278 def self.included(base) base.extend(MapClassMethods) end |
Instance Method Details
#add_annotation(annotation) ⇒ Object
98 99 100 101 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 98 def add_annotation(annotation) @promotion_annotation_data << MapScreenAnnotation.new(annotation) self.view.addAnnotation @promotion_annotation_data.last end |
#add_annotations(annotations) ⇒ Object
103 104 105 106 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 103 def add_annotations(annotations) @promotion_annotation_data = Array(annotations).map{|a| MapScreenAnnotation.new(a)} self.view.addAnnotations @promotion_annotation_data end |
#annotation_view(map_view, annotation) ⇒ Object
115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 115 def annotation_view(map_view, annotation) return if annotation.is_a? MKUserLocation params = annotation.params identifier = params[:identifier] if view = map_view.dequeueReusableAnnotationViewWithIdentifier(identifier) view.annotation = annotation else # Set the pin properties if params[:image] view = MKAnnotationView.alloc.initWithAnnotation(annotation, reuseIdentifier:identifier) else view = MKPinAnnotationView.alloc.initWithAnnotation(annotation, reuseIdentifier:identifier) end end view.image = params[:image] if view.respond_to?("image=") && params[:image] view.animatesDrop = params[:animates_drop] if view.respond_to?("animatesDrop=") view.pinColor = params[:pin_color] if view.respond_to?("pinColor=") view.canShowCallout = params[:show_callout] if view.respond_to?("canShowCallout=") if params[:left_accessory] view.leftCalloutAccessoryView = params[:left_accessory] end if params[:right_accessory] view.rightCalloutAccessoryView = params[:right_accessory] end if params[:action] = params[:action_button_type] || UIButtonTypeDetailDisclosure = UIButton.() .addTarget(self, action: params[:action], forControlEvents:UIControlEventTouchUpInside) view.rightCalloutAccessoryView = end view end |
#annotations ⇒ Object
74 75 76 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 74 def annotations @promotion_annotation_data end |
#center ⇒ Object
33 34 35 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 33 def center self.view.centerCoordinate end |
#center=(params = {}) ⇒ Object
37 38 39 40 41 42 43 44 45 46 47 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 37 def center=(params={}) PM.logger.error "Missing #:latitude property in call to #center=." unless params[:latitude] PM.logger.error "Missing #:longitude property in call to #center=." unless params[:longitude] params[:animated] ||= true # Set the new region self.view.setCenterCoordinate( CLLocationCoordinate2D.new(params[:latitude], params[:longitude]), animated:params[:animated] ) end |
#check_annotation_data ⇒ Object
19 20 21 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 19 def check_annotation_data PM.logger.error "Missing #annotation_data method in MapScreen #{self.class.to_s}." unless self.respond_to?(:annotation_data) end |
#clear_annotations ⇒ Object
108 109 110 111 112 113 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 108 def clear_annotations @promotion_annotation_data.each do |a| self.view.removeAnnotation(a) end @promotion_annotation_data = [] end |
#deselect_annotations(animated = false) ⇒ Object
90 91 92 93 94 95 96 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 90 def deselect_annotations(animated=false) unless selected_annotations.nil? selected_annotations.each do |annotation| self.view.deselectAnnotation(annotation, animated:animated) end end end |
#hide_user_location ⇒ Object
53 54 55 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 53 def hide_user_location set_show_user_location false end |
#look_up_address(args = {}, &callback) ⇒ Object
222 223 224 225 226 227 228 229 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 222 def look_up_address(args={}, &callback) args[:address] = args if args.is_a? String # Assume if a string is passed that they want an address geocoder = CLGeocoder.new return geocoder.geocodeAddressDictionary(args[:address], completionHandler: callback) if args[:address].is_a?(Hash) return geocoder.geocodeAddressString(args[:address].to_s, completionHandler: callback) unless args[:region] return geocoder.geocodeAddressString(args[:address].to_s, inRegion:args[:region].to_s, completionHandler: callback) if args[:region] end |
#map ⇒ Object Also known as: mapview
28 29 30 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 28 def map self.view end |
#mapView(map_view, didUpdateUserLocation: userLocation) ⇒ Object
Cocoa touch methods #################
232 233 234 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 232 def mapView(map_view, viewForAnnotation:annotation) annotation_view(map_view, annotation) end |
#region(params) ⇒ Object
211 212 213 214 215 216 217 218 219 220 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 211 def region(params) return nil unless params.is_a? Hash params[:coordinate] = CLLocationCoordinate2D.new(params[:coordinate][:latitude], params[:coordinate][:longitude]) if params[:coordinate].is_a? Hash params[:span] = MKCoordinateSpanMake(params[:span][0], params[:span][1]) if params[:span].is_a? Array if params[:coordinate] && params[:span] MKCoordinateRegionMake( params[:coordinate], params[:span] ) end end |
#screen_setup ⇒ Object
4 5 6 7 8 9 10 11 12 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 4 def screen_setup self.view = nil self.view = MKMapView.alloc.initWithFrame(self.view.bounds) self.view.delegate = self check_annotation_data @promotion_annotation_data = [] set_up_start_position end |
#select_annotation(annotation, animated = true) ⇒ Object
78 79 80 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 78 def select_annotation(annotation, animated=true) self.view.selectAnnotation(annotation, animated:animated) end |
#select_annotation_at(annotation_index, animated = true) ⇒ Object
82 83 84 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 82 def select_annotation_at(annotation_index, animated=true) select_annotation(annotations[annotation_index], animated:animated) end |
#selected_annotations ⇒ Object
86 87 88 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 86 def selected_annotations self.view.selectedAnnotations end |
#set_region(region, animated = true) ⇒ Object
207 208 209 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 207 def set_region(region, animated=true) self.view.setRegion(region, animated:animated) end |
#set_show_user_location(show) ⇒ Object
57 58 59 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 57 def set_show_user_location(show) self.view.showsUserLocation = show end |
#set_start_position(params = {}) ⇒ Object
154 155 156 157 158 159 160 161 162 163 164 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 154 def set_start_position(params={}) params[:latitude] ||= 37.331789 params[:longitude] ||= -122.029620 params[:radius] ||= 10 meters_per_mile = 1609.344 initialLocation = CLLocationCoordinate2D.new(params[:latitude], params[:longitude]) region = MKCoordinateRegionMakeWithDistance(initialLocation, params[:radius] * meters_per_mile, params[:radius] * meters_per_mile) set_region(region, animated:false) end |
#set_up_start_position ⇒ Object
166 167 168 169 170 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 166 def set_up_start_position if self.class.respond_to?(:get_start_position) && self.class.get_start_position self.set_start_position self.class.get_start_position_params end end |
#show_user_location ⇒ Object
49 50 51 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 49 def show_user_location set_show_user_location true end |
#showing_user_location? ⇒ Boolean
61 62 63 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 61 def showing_user_location? self.view.showsUserLocation end |
#type ⇒ Object
Cocoa touch Ruby counterparts #################
246 247 248 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 246 def type map.mapType end |
#type=(type) ⇒ Object
250 251 252 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 250 def type=(type) map.mapType = type end |
#update_annotation_data ⇒ Object
23 24 25 26 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 23 def update_annotation_data clear_annotations add_annotations annotation_data end |
#user_location ⇒ Object
65 66 67 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 65 def user_location self.view.userLocation.location.nil? ? nil : self.view.userLocation.location.coordinate end |
#view_will_appear(animated) ⇒ Object
14 15 16 17 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 14 def view_will_appear(animated) super update_annotation_data end |
#zoom_to_fit_annotations(animated = true) ⇒ Object
TODO: Why is this so complex?
173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 173 def zoom_to_fit_annotations(animated=true) #Don't attempt the rezoom of there are no pins return if annotations.count == 0 #Set some crazy boundaries topLeft = CLLocationCoordinate2D.new(-90, 180) bottomRight = CLLocationCoordinate2D.new(90, -180) #Find the bounds of the pins annotations.each do |a| topLeft.longitude = [topLeft.longitude, a.coordinate.longitude].min topLeft.latitude = [topLeft.latitude, a.coordinate.latitude].max bottomRight.longitude = [bottomRight.longitude, a.coordinate.longitude].max bottomRight.latitude = [bottomRight.latitude, a.coordinate.latitude].min end #Find the bounds of all the pins and set the map_view coord = CLLocationCoordinate2D.new( topLeft.latitude - (topLeft.latitude - bottomRight.latitude) * 0.5, topLeft.longitude + (bottomRight.longitude - topLeft.longitude) * 0.5 ) # Add some padding to the edges span = MKCoordinateSpanMake( ((topLeft.latitude - bottomRight.latitude) * 1.075).abs, ((bottomRight.longitude - topLeft.longitude) * 1.075).abs ) region = MKCoordinateRegionMake(coord, span) fits = self.view.regionThatFits(region); set_region(fits, animated:animated) end |
#zoom_to_user(radius = 0.05, animated = true) ⇒ Object
69 70 71 72 |
# File 'lib/ProMotion/map/map_screen_module.rb', line 69 def zoom_to_user(radius = 0.05, animated=true) show_user_location unless showing_user_location? set_region(MKCoordinateRegionMake(user_location, [radius, radius]), animated) end |